<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is a proprietary licensed product. 
 * For more information see License.txt in the plugin folder.

 * ---
 * Copyright (c) 2018, Ebenezer Obasi
 * All rights reserved.
 * info@eobasi.com.

 * Redistribution and use in source and binary forms, with or without modification, are not permitted provided.

 * This plugin should be bought from the developer. For details contact info@eobasi.com.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 *  Helpcenter Service.
 * 
 * @author Ebenezer Obasi <info@eobasi.com>
 * @package ow_plugins.helpcenter.bol
 * @since 1.0
 */
class HELPCENTER_BOL_Service
{
	const COUNT_PER_PAGE = 20;
	const COUNT_FIRST = 0;
	
	const FEATURED_TAG = 'featured';
	
    /**
     * Singleton instance.
     *
     * @var HELPCENTER_BOL_Service
     */
    private static $classInstance;

    /**
     * Returns an instance of class (singleton pattern implementation).
     *
     * @return HELPCENTER_BOL_Service
     */
	public static function getInstance()
	{
		if( self::$classInstance === null )
		{
			self::$classInstance = new self();
		}

		return self::$classInstance;
	}

    /**
     * @var HELPCENTER_BOL_CategoryDao
     */
	private $categoryDao;
    /**
     * @var HELPCENTER_BOL_TopicDao
     */
	private $topicDao;
    /**
     * @var OW_Config
     */	
	public $config;

    /**
     * Constructor.
     */
	private function __construct( )
	{
		$this->lang = OW::getLanguage();

		$this->categoryDao = HELPCENTER_BOL_CategoryDao::getInstance();
		$this->topicDao = HELPCENTER_BOL_TopicDao::getInstance();
		
		$this->config = (object)OW::getConfig()->getValues( 'helpcenter' );
	}
	
	/**
     * Search helpcenter topics with provided term.
     *
     * @param string $term
     * @param integer $first
     * @param integer $count
     * @return array<HELPCENTER_BOL_Topic>
     */
    public function searchTopic( $term, $first = null, $count = null )
    {
        $results = $this->topicDao->search( $term, $first, $count );
		
		foreach( $results as $result )
		{
			$result->url = $this->itemUrl( $result->id, 'topic' );
		}
		
		return $results;
    }
	
	public function findTopicsByTag( $tag, $first, $count )
	{
		$tagService = BOL_TagService::getInstance();
		
		$tags = $tagService->findEntityListByTag( 'helpcenter', $tag, $first, $count);
		
		return $this->topicDao->findByIdList($tags);
	}

	public function isGrandChild( $categoryId )
	{
		$category = $this->findCategory( $categoryId, true );

		if( !empty( $category->grandParent ) && (int)$category->grandParent->parent == 0 )
		{
			return true;
		}

		return false;
	}

	public function categoryTopics( $categoryId )
	{
		return $this->topicDao->findCategoryTopics( $categoryId );
	}

    /**
     * Saves topic dto.
     *
     * @param HELPCENTER_BOL_Topic $topic
     */
	public function saveTopic( $topic )
	{
		return $this->topicDao->saveTopic( $topic );
	}

	public function findTopic( $id )
	{
		return $this->topicDao->findById( $id );
	}

    /**
     * Deletes topic.
     *
     * @param integer $topicId
     */
	public function deleteTopic( $topicId )
	{
		return $this->topicDao->deleteById( $topicId );
	}

	public function findAllTopic()
	{
		return $this->topicDao->findAll();
	}

	public function saveCategory( $category )
	{
		$this->categoryDao->save( $category );
	}

	public function findAllCategory( $segments = false )
	{
		if( $segments === false )
		{
			return $this->categoryDao->findAll();
		}

		$categories = $this->findParentCategories();

		foreach( $categories as $category )
		{
			$children = $this->findCategoryChildren( $category->getId( ) );
			$category->children = $children;

			foreach($children as $child)
			{
				$granChild = $this->findCategoryChildren( $child->getId( ) );
				$child->children = $granChild;
			}
		}

		return $categories;
	}

	public function findCategory( $id, $segments = false )
	{
		if( $segments === false )
		{
			return $this->categoryDao->findById( $id );
		}

		$category = $this->categoryDao->findById( $id );
		$category->section = $this->categoryDao->findById( $category->getParent() );
		$category->grandParent = null;

		if( !empty( $category->section ) )
		{
			$category->grandParent = $this->categoryDao->findById( $category->section->getParent() );
		}

		return $category;
	}

	public function findParentCategories()
	{
		return $this->categoryDao->parentCategories();
	}

	public function findCategoryChildren( $categoryId )
	{
		return $this->categoryDao->categoryChildren( $categoryId );
	}

    /**
     * Deletes category.
     *
     * @param integer $categoryId
     */
	public function deleteCategory( $categoryId )
	{
		return $this->categoryDao->deleteById( $categoryId );
	}


    /**
     * Returns menu component
     *
     * @return BASE_CMP_ContentMenu
     */
    public function getAdminMenu()
    {
        $language = OW::getLanguage();

		$menu = new BASE_CMP_ContentMenu();

		$item = new BASE_MenuItem();
		$item->setLabel($this->lang->text('helpcenter', 'menu_item_admin_index'));
		$item->setUrl(OW::getRouter()->urlForRoute('helpcenter_admin_index'));
		$item->setKey('index');
		$item->setIconClass('ow_ic_help');
		$item->setOrder(1);
		$menu->addElement($item);

		$item = new BASE_MenuItem();
		$item->setLabel($this->lang->text('helpcenter', 'menu_item_admin_add'));
		$item->setUrl(OW::getRouter()->urlForRoute('helpcenter_admin_add_new'));
		$item->setKey('add');
		$item->setIconClass('ow_ic_write');
		$item->setOrder(2);
		$menu->addElement($item);

		$item = new BASE_MenuItem();
		$item->setLabel($this->lang->text('helpcenter', 'menu_item_admin_category'));
		$item->setUrl(OW::getRouter()->urlForRoute('helpcenter_admin_category'));
		$item->setKey('category');
		$item->setIconClass('ow_ic_folder');
		$item->setOrder(3);
		$menu->addElement($item);

		$item = new BASE_MenuItem();
		$item->setLabel($this->lang->text('helpcenter', 'menu_item_admin_settings'));
		$item->setUrl(OW::getRouter()->urlForRoute('helpcenter_admin_settings'));
		$item->setKey('setting');
		$item->setIconClass('ow_ic_gear_wheel');
		$item->setOrder(4);
		$menu->addElement($item);

        return $menu;
    }
	
    /**
     * Returns serialized custom theme css array.
     * 
     * @param string $css
     * @return serialize/array  
     */
	public function setThemeCustomCss( $css )
	{
		$theme = $this->getCurrentTheme();
		$themeCss = array();
		
		$configCss = unserialize( $this->config->css );
		$configCss = is_array( $configCss ) ? $configCss : array();
		
		$themeCss[$theme] = $css;		
		$cssStorage = array_merge($configCss, $themeCss);
		
		return serialize( $cssStorage );
	}
	
    /**
     * Returns helpcenter custom css.
     * 
     * @return string  
     */
	public function getThemeCustomCss()
	{
		$theme = $this->getCurrentTheme();
		$css = unserialize( $this->config->css );
		
		return isset( $css[$theme] ) ? $css[$theme] : '';
	}

    /**
     * Returns array of helpcenter themes.
     * 
     * @return array  
     */
	public function getThemes()
	{		
		$themeList = array();
		$storage = new BASE_CLASS_FileStorage();

		$themes = $storage->getFileNameList( $this->themesDir() );

		foreach( $themes as $theme )
		{
			if( $storage->isDir( $theme ) )
			{
				$theme = basename( $theme );
				$themeList[$theme] = $this->fileRealName( $theme );
			}
		}

		return $themeList;
	}

    /**
     * Returns current helpcenter theme name.
     * 
     * @return string  
     */
	public function getCurrentTheme()
	{
		return $this->config->theme;
	}

    /**
     * Returns theme directory url for current helpcenter theme.
     * 
     * @return string  
     */
	public function getCurrentThemeUrl()
	{
		return $this->themesUrl() . $this->getCurrentTheme() . '/';
	}

    /**
     * Returns helpcenter theme directory.
     * 
     * @return string  
     */
	public function themesDir()
	{
		return OW::getPluginManager()->getPlugin('helpcenter')->getStaticDir() . 'themes' . DS;
	}
	
    /**
     * Returns helpcenter themes directory url.
     * 
     * @return string  
     */
	public function themesUrl()
	{
		return $this->staticUrl() . 'themes/';
	}
	
    /**
     * Returns helpcenter static url.
     * 
     * @return string  
     */
	public function staticUrl()
	{
		return OW::getPluginManager()->getPlugin('helpcenter')->getStaticUrl();
	}
	
    /**
     * Returns real name for a given file.
     * 
     * @param string $file
     * @return string  
     */
	public function fileRealName( $file )
	{
		$file = str_replace( "-", " ", $file );
		$file = str_replace( "_"," ", $file );
		$file = ucwords( $file );
		
		return $file;
	}

	public function itemUrl( $id, $type )
	{
		$url = array(
			'delete' => OW::getRouter()->urlForRoute('helpcenter_admin_delete', array('type' => $type, 'id' => $id)),
			'view' => '',
			'edit' => '',
		);

		switch( $type )
		{
			case 'category':
				$url['view'] = OW::getRouter()->urlForRoute('helpcenter_view_board', array('catId' => $id));
				$url['edit'] =  OW::getRouter()->urlForRoute('helpcenter_admin_category', array('id' => $id));
				break;
			case 'topic':
				$url['view'] = OW::getRouter()->urlForRoute('helpcenter_view_topic', array('id' => $id));
				$url['edit'] = OW::getRouter()->urlForRoute('helpcenter_admin_add', array('id' => $id));
				break;
		}

		return $url;
	}
}